==> crow-2.11.0.tar.bz2: crow-2.11.0/crow/util.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_UTIL_H
#define CROW_UTIL_H

#include "crow.h"

namespace Crow {

#define Check(expr) if(!(expr)) CheckFailed(#expr, __FILE__, __LINE__); else

void CheckFailed(const char * expr, const char * file, unsigned line);
void FatalError(const std::string & message);
std::string GetTypeName(const std::type_info & ti);
TypeId FindType(const std::string typeName);
PaletteEntry * GetEntry(TypeId type);
void DumpParents(const gchar * name);
void NoOp();
IdPath GetParentPath(const IdPath & path);
const char * GetData(Glib::Object * object, const std::string & key);
void SetData(Glib::Object * object, const std::string & key, const char * value);
GType GetGType(Glib::Object * object);
Point Transform(const Point & p, Gtk::Widget * src, Gtk::Widget * dest);
void SplitNumber(const std::string & name, std::string * str, std::string * num);
void SplitNumber(const std::string & name, std::string * str, int * num);
bool LessSplit(const std::string & name1, const std::string & name2);
std::string RightPart(const std::string & str, char ch);
std::vector<Glib::ustring> Split(const Glib::ustring & str, gunichar ch, int count=0);
Glib::ustring XmlEncode(const Glib::ustring & str, bool attr=false);
bool ValidName(const std::string & name);
void DrawGrid(PGdkWindow window, PGtkStyle style, bool squares=false);
void RaiseError(const Glib::ustring & msg);
int GetRefCount(const Glib::ObjectBase * object);
void ContainerClear(Gtk::Container * container);
void ContainerRemove(Gtk::Container * container, Gtk::Widget * widget);
void BinRemove(Gtk::Bin * bin);
NodeSet ToNodeSet(const Nodes & nodes);
Gdk::Color GetWhite(PGtkStyle style);
Gdk::Color GetBlack(PGtkStyle style);
Gdk::Color GetLight(PGtkStyle style);
Gdk::Color GetDark(PGtkStyle style);
std::string IdPathToString(const IdPath & path, bool dots=false);
void SortByName(Nodes * nodes);
void SetColumnWidth(Gtk::TreeViewColumn * column, int width);
std::string Trim(const std::string & str);
bool Equal(const PObject & v1, const PObject & v2);
Settings HashTableToSettings(GHashTable * table);
void SettingsToHashTable(const Settings & settings, GHashTable * table);
void PrepareGlibObject(Glib::Object * object);
int TypeDist(GType sub, GType type);

template <class T> T * GetEntry(TypeId type) {
	return dynamic_cast<T *>(GetEntry(type));
}

template <class U, class V> U safe_cast(const V & v) {
	U u= (U)v;
	Check((V)u == v);
	return u;
}

template <class T> Glib::ustring ToString(const T & value) {
	std::ostringstream oss;
	oss << value;
	return oss.str();
}

template <class T> T FromString(const Glib::ustring & str) {
	std::istringstream iss(str);
	T value;
	iss >> value;
	if(iss.fail())
		RaiseError("FromString() failed");
	return value;
}

template <> inline Glib::ustring ToString<Glib::ustring>(const Glib::ustring & value) { return value; }
template <> inline Glib::ustring FromString<Glib::ustring>(const Glib::ustring & str) { return str; }

template <> inline Glib::ustring ToString<std::string>(const std::string & value) { return value; }
template <> inline std::string FromString<std::string>(const Glib::ustring & str) { return str; }

template <> Glib::ustring ToString<bool>(const bool & value);
template <> bool FromString<bool>(const Glib::ustring & str);

template <> Glib::ustring ToString<Gdk::Color>(const Gdk::Color & value);
template <> Gdk::Color FromString<Gdk::Color>(const Glib::ustring & str);

template <> Glib::ustring ToString<Gtk::StockID>(const Gtk::StockID & value);
template <> Gtk::StockID FromString<Gtk::StockID>(const Glib::ustring & str);

template <class T> T * GetPtr(const Glib::RefPtr<T> & refptr) {
	return refptr.operator->();
}

template <class T> bool operator < (const Glib::RefPtr<T> & p1, const Glib::RefPtr<T> & p2) {
	return GetPtr(p1) < GetPtr(p2);
}

template <class C1, class C2> bool Begins(const C1 & c1, const C2 & c2) {
	if(c1.size() < c2.size())
		return false;
	return std::equal(c2.begin(), c2.end(), c1.begin());
}

template <class C1, class C2> bool Ends(const C1 & c1, const C2 & c2) {
	if(c1.size() < c2.size())
		return false;
	return std::equal(c2.rbegin(), c2.rend(), c1.rbegin());
}

template <class T> typename T::iterator EraseAndNext(T & cont, const typename T::iterator & iter) {
	typename T::iterator next= iter;
	++next;
	cont.erase(iter);
	return next;
}

template <class T> T Next(const T & iter) {
	T next= iter;
	++next;
	return next;
}

template <class T> T Prev(const T & iter) {
	T prev= iter;
	--prev;
	return prev;
}

template <class T> bool ListsEqualSorted(const T & cont1, const T & cont2) {
	if(cont1.size() != cont2.size())
		return false;
	T c1= cont1;
	T c2= cont2;
	c1.sort();
	c2.sort();
	return c1 == c2;
}

template <class T> bool FindAndErase(T & cont, const typename T::value_type & elem) {
	typename T::iterator iter= std::find(cont.begin(), cont.end(), elem);
	if(iter != cont.end()) {
		cont.erase(iter);
		return true;
	} else
		return false;
}

template <class T> bool PushIfAbsent(T & cont, const typename T::value_type & elem) {
	typename T::iterator iter= std::find(cont.begin(), cont.end(), elem);
	if(iter == cont.end()) {
		cont.push_back(elem);
		return true;
	} else
		return false;
}

template <class T> T ExtractCppValue(const Glib::ValueBase & val) {
	Glib::Value<T> tmp;
	tmp.init(Glib::Value<T>::value_type());
	Check(g_value_type_compatible(G_VALUE_TYPE(val.gobj()), G_VALUE_TYPE(tmp.gobj())));
	g_value_copy(val.gobj(), tmp.gobj());
	return tmp.get();
}

template <class T> Glib::ValueBase MakeGValue(const T & val) {
	Glib::Value<T> value;
	value.init(Glib::Value<T>::value_type());
	value.set(val);
	return value;
}

template <class T> T * GetPtrRef(const Glib::RefPtr<T> & refptr) {
	T * t= GetPtr(refptr);
	if(t)
		t->reference();
	return t;
}

template <class T> void ChangeAttribute(Pango::AttrList * attributes, const T & attribute) {
	T attr= attribute;
	attr.set_start_index(0);
	attr.set_end_index(G_MAXINT);
	attributes->change(attr);
}

class RuntimeError : public Glib::Error {

public:
	RuntimeError(const Glib::ustring & msg);
	RuntimeError();

};

class ObjectBase {

public:
	virtual ~ObjectBase() { }

};

class Object : virtual public ObjectBase, virtual public sigc::trackable {

friend class CrowInit;

public:
	Object();
	~Object();

	void reference();
	void unreference();

	virtual bool equals(PObject po) const;

private:
	int refcounter;

};

template <class T> Glib::RefPtr<T> MakeRefPtr(T * t) {
	if(t)
		t->reference();
	return Glib::RefPtr<T>(t);
}

template <class T> Glib::RefPtr<T> NewRefPtr(T * t=new T) {
	if(Glib::Object * obj= dynamic_cast<Glib::Object *>(t))
		PrepareGlibObject(obj);
	else if(Object * obj= dynamic_cast<Object *>(t))
		obj->reference();
	return Glib::RefPtr<T>(t);
}

class UniChar {

public:
	UniChar() { unichar= gunichar(); }
	UniChar(gunichar ch) { unichar= ch; }
	operator gunichar () const { return unichar; }

private:
	gunichar unichar;

};

template <> Glib::ustring ToString<UniChar>(const UniChar & value);
template <> UniChar FromString<UniChar>(const Glib::ustring & str);

template <> UniChar ExtractCppValue<UniChar>(const Glib::ValueBase & val);
template <> Glib::ValueBase MakeGValue<UniChar>(const UniChar & val);

class Translatable {

public:
	Translatable(const Glib::ustring & meta=Glib::ustring());
	Translatable(bool trans, const Glib::ustring & prefix, const Glib::ustring & comments);

	bool getTranslate() const { return trans; }
	const Glib::ustring * getPrefix() const { return &prefix; }
	const Glib::ustring * getComments() const { return &comments; }
	bool isValid() const { return valid; }
	
	Glib::ustring getMeta() const;

protected:
	void testValid();

private:
	bool trans;
	Glib::ustring prefix;
	Glib::ustring comments;
	bool valid;

};

template <class Key> class Pool {

public:
	Pool(int cap) { capacity= cap; }

	PObject get(const Key & key) {
		typename Entries::iterator it= find(key);
		return it!=entries.end() ? it->second : PObject();
	}

	template <class T> Glib::RefPtr<T> get(const Key & key) {
		typename Entries::iterator it= find(key);
		return it!=entries.end() ? Glib::RefPtr<T>::cast_dynamic(it->second) : Glib::RefPtr<T>();
	}

	void set(const Key & key, PObject object) {
		typename Entries::iterator it= find(key);
		entries.push_back(std::make_pair(key, object));
		if(it==entries.end()) {
			if((int)entries.size() > capacity)
				entries.pop_front();
		} else
			entries.erase(it);
	}

protected:
	typedef std::list<std::pair<Key, PObject> > Entries;

	typename Entries::iterator find(const Key & key) {
		for(typename Entries::iterator it=entries.begin(); it!=entries.end(); ++it)
			if(it->first==key)
				return it;
		return entries.end();
	}

private:
	int capacity;
	Entries entries;

};

struct Point {
	int x, y;
	Point();
	Point(int x, int y);
	bool inside(const Rectangle & p) const;
	Point operator + (const Point & p) const;
	Point operator - (const Point & p) const;
	bool operator == (const Point & p) const;
	Point operator - () const;
};

template <> Glib::ustring ToString<Point>(const Point & value);
template <> Point FromString<Point>(const Glib::ustring & str);

struct Rectangle {
	int x, y, w, h;
	Rectangle();
	Rectangle(int x, int y, int w, int h);
	Rectangle(const Point & point, const Point & size);
	void grow(int dx, int dy, int dw, int dh);
	void grow(int dx, int dy, int dw, int dh, int minw, int minh);
	Point point() const { return Point(x, y); }
	Point size() const { return Point(w, h); }
	bool empty() const;
	Rectangle(const Gdk::Rectangle & rect);
	operator Gdk::Rectangle () const;
	bool operator == (const Rectangle & r) const;
	void growSegment(int * a, int * b, int da, int db, int min);
};

struct Border {
	int left, right, top, bottom;
	Border();
	bool operator == (const Border & b) const;
	operator Gtk::Border () const;
};

template <> Glib::ustring ToString<Border>(const Border & value);
template <> Border FromString<Border>(const Glib::ustring & str);

struct Emitter {
	std::string signame;
	std::string mapname;
	bool after;
	Emitter() { after= true; }
	Emitter(const std::string & sname, const std::string & mname, bool after);
	bool operator == (const Emitter & e) const;
};

template <> Glib::ustring ToString<Emitter>(const Emitter & value);
template <> Emitter FromString<Emitter>(const Glib::ustring & str);

class Id {

public:
	Id() { number= -1; }
	Id(const std::string & str) { SplitNumber(str, &string, &number); }	
	Id(int num) { number= num; }

	std::string toString() const { return string + (number==-1 ? Glib::ustring() : ToString(number)); }

	bool operator == (const Id & id) const { return string==id.string && number==id.number; }
	bool operator < (const Id & id) const { return string==id.string ? number < id.number : string < id.string; }

private:
	std::string string;
	int number;

};

class Parser {

public:
	Parser();
	virtual ~Parser() { }

	static void parse(Parser * parser, const Glib::ustring & data);

protected:
	typedef Glib::Markup::Parser::AttributeMap AttributeMap;

	virtual void onStartElement(const Glib::ustring & name, const AttributeMap & properties) { }
	virtual void onEndElement(const Glib::ustring & name) { }
	virtual void onText(const Glib::ustring & text) { }
	virtual void onPassthrough(const Glib::ustring & text) { }

private:

	class GlibParser : public Glib::Markup::Parser {

	public:
		GlibParser(::Crow::Parser * parser) { this->parser= parser; }

	protected:		
		void raiseError(const Glib::Exception & x);
		
		void on_start_element(Glib::Markup::ParseContext & context, const Glib::ustring & name, const AttributeMap & properties);
		void on_end_element(Glib::Markup::ParseContext & context, const Glib::ustring & name);
		void on_text(Glib::Markup::ParseContext & context, const Glib::ustring & text);
		void on_passthrough(Glib::Markup::ParseContext & context, const Glib::ustring & text);

	private:
		::Crow::Parser * parser;

	};

	GlibParser glibParser;

};

class UiElementExtract : public Parser {

public:
	typedef std::map<std::string, std::string> emap;
	
	emap extract(const Glib::ustring & str);

protected:
	void onStartElement(const Glib::ustring & name, const AttributeMap & properties);
	void onEndElement(const Glib::ustring & name) { --depth; }

private:
	int depth;
	emap result;

};

}

#endif

==> crow-2.11.3.tar.bz2: crow-2.11.3/crow/util.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2008 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_UTIL_H
#define CROW_UTIL_H

#include "crow.h"

namespace Crow {

#define Check(expr) ((expr) ? NoObj() : (CheckFailed(#expr, __FILE__, __LINE__), NoObj()))

struct NoObj { };

void CheckFailed(const char * expr, const char * file, unsigned line);
void FatalError(const std::string & message);
std::string GetTypeName(const std::type_info & ti);
TypeId FindType(const std::string typeName);
PaletteEntry * GetEntry(TypeId type);
void DumpParents(const gchar * name);
void NoOp();
IdPath GetParentPath(const IdPath & path);
const char * GetData(Glib::Object * object, const std::string & key);
void SetData(Glib::Object * object, const std::string & key, const char * value);
GType GetGType(Glib::Object * object);
Point Transform(const Point & p, Gtk::Widget * src, Gtk::Widget * dest);
void SplitNumber(const std::string & name, std::string * str, std::string * num);
void SplitNumber(const std::string & name, std::string * str, int * num);
bool LessSplit(const std::string & name1, const std::string & name2);
std::string RightPart(const std::string & str, char ch);
std::vector<Glib::ustring> Split(const Glib::ustring & str, gunichar ch, int count=0);
Glib::ustring XmlEncode(const Glib::ustring & str, bool attr=false);
bool ValidName(const std::string & name);
void DrawGrid(PGdkWindow window, PGtkStyle style, bool squares=false);
void RaiseError(const Glib::ustring & msg);
int GetRefCount(const Glib::ObjectBase * object);
void ContainerClear(Gtk::Container * container);
void ContainerRemove(Gtk::Container * container, Gtk::Widget * widget);
void BinRemove(Gtk::Bin * bin);
NodeSet ToNodeSet(const Nodes & nodes);
Gdk::Color GetWhite(PGtkStyle style);
Gdk::Color GetBlack(PGtkStyle style);
Gdk::Color GetLight(PGtkStyle style);
Gdk::Color GetDark(PGtkStyle style);
std::string IdPathToString(const IdPath & path, bool dots=false);
void SortByName(Nodes * nodes);
void SetColumnWidth(Gtk::TreeViewColumn * column, int width);
Glib::ustring Trim(const Glib::ustring & str);
bool Equal(const PObject & v1, const PObject & v2);
Settings HashTableToSettings(GHashTable * table);
void SettingsToHashTable(const Settings & settings, GHashTable * table);
void PrepareGlibObject(Glib::Object * object);
int TypeDist(GType sub, GType type);
Glib::ustring Substitute(const Glib::ustring & str, const Glib::ustring & key, const Glib::ustring & val);
Glib::ustring Underscores(const Glib::ustring & str);
Bits Addition(Bits * v, Bits f);
Bits Subtraction(Bits * v, Bits f);
bool Includes(Bits v, Bits f);
bool Intersect(Bits v, Bits f);

template <class T> T * GetEntry(TypeId type) {
	return dynamic_cast<T *>(GetEntry(type));
}

template <class U, class V> U safe_cast(const V & v) {
	U u= (U)v;
	Check((V)u == v);
	return u;
}

template <class T> Glib::ustring ToString(const T & value) {
	std::ostringstream oss;
	oss << value;
	return oss.str();
}

template <class T> T FromString(const Glib::ustring & str) {
	std::istringstream iss(str);
	T value;
	iss >> value;
	if(iss.fail())
		RaiseError("FromString() failed");
	return value;
}

template <> inline Glib::ustring ToString<Glib::ustring>(const Glib::ustring & value) { return value; }
template <> inline Glib::ustring FromString<Glib::ustring>(const Glib::ustring & str) { return str; }

template <> inline Glib::ustring ToString<std::string>(const std::string & value) { return value; }
template <> inline std::string FromString<std::string>(const Glib::ustring & str) { return str; }

template <> Glib::ustring ToString<bool>(const bool & value);
template <> bool FromString<bool>(const Glib::ustring & str);

template <> Glib::ustring ToString<Gdk::Color>(const Gdk::Color & value);
template <> Gdk::Color FromString<Gdk::Color>(const Glib::ustring & str);

template <> Glib::ustring ToString<Gtk::StockID>(const Gtk::StockID & value);
template <> Gtk::StockID FromString<Gtk::StockID>(const Glib::ustring & str);

template <class T> T * GetPtr(const Glib::RefPtr<T> & refptr) {
	return refptr.operator->();
}

template <class T> bool operator < (const Glib::RefPtr<T> & p1, const Glib::RefPtr<T> & p2) {
	return GetPtr(p1) < GetPtr(p2);
}

template <class C1, class C2> bool Begins(const C1 & c1, const C2 & c2) {
	if(c1.size() < c2.size())
		return false;
	return std::equal(c2.begin(), c2.end(), c1.begin());
}

template <class C1, class C2> bool Ends(const C1 & c1, const C2 & c2) {
	if(c1.size() < c2.size())
		return false;
	return std::equal(c2.rbegin(), c2.rend(), c1.rbegin());
}

template <class T> typename T::iterator EraseAndNext(T & cont, const typename T::iterator & iter) {
	typename T::iterator next= iter;
	++next;
	cont.erase(iter);
	return next;
}

template <class T> T Next(const T & iter) {
	T next= iter;
	++next;
	return next;
}

template <class T> T Prev(const T & iter) {
	T prev= iter;
	--prev;
	return prev;
}

template <class T> bool ListsEqualSorted(const T & cont1, const T & cont2) {
	if(cont1.size() != cont2.size())
		return false;
	T c1= cont1;
	T c2= cont2;
	c1.sort();
	c2.sort();
	return c1 == c2;
}

template <class T> bool FindAndErase(T & cont, const typename T::value_type & elem) {
	typename T::iterator iter= std::find(cont.begin(), cont.end(), elem);
	if(iter != cont.end()) {
		cont.erase(iter);
		return true;
	} else
		return false;
}

template <class T> bool PushIfAbsent(T & cont, const typename T::value_type & elem) {
	typename T::iterator iter= std::find(cont.begin(), cont.end(), elem);
	if(iter == cont.end()) {
		cont.push_back(elem);
		return true;
	} else
		return false;
}

template <class T> T ExtractCppValue(const Glib::ValueBase & val) {
	Glib::Value<T> tmp;
	tmp.init(Glib::Value<T>::value_type());
	Check(g_value_type_compatible(G_VALUE_TYPE(val.gobj()), G_VALUE_TYPE(tmp.gobj())));
	g_value_copy(val.gobj(), tmp.gobj());
	return tmp.get();
}

template <class T> Glib::ValueBase MakeGValue(const T & val) {
	Glib::Value<T> value;
	value.init(Glib::Value<T>::value_type());
	value.set(val);
	return value;
}

template <class T> T * GetPtrRef(const Glib::RefPtr<T> & refptr) {
	T * t= GetPtr(refptr);
	if(t)
		t->reference();
	return t;
}

template <class T> void ChangeAttribute(Pango::AttrList * attributes, const T & attribute) {
	T attr= attribute;
	attr.set_start_index(0);
	attr.set_end_index(G_MAXINT);
	attributes->change(attr);
}

class RuntimeError : public Glib::Error {

public:
	RuntimeError(const Glib::ustring & msg);
	RuntimeError();

};

class ObjectBase {

public:
	virtual ~ObjectBase() { }

};

class Object : virtual public ObjectBase, virtual public sigc::trackable {

friend class CrowInit;

public:
	Object();
	~Object();

	void reference();
	void unreference();

	virtual bool equals(PObject po) const;

private:
	int refcounter;

};

template <class T> Glib::RefPtr<T> MakeRefPtr(T * t) {
	if(t)
		t->reference();
	return Glib::RefPtr<T>(t);
}

template <class T> Glib::RefPtr<T> NewRefPtr(T * t=new T) {
	if(Glib::Object * obj= dynamic_cast<Glib::Object *>(t))
		PrepareGlibObject(obj);
	else if(Object * obj= dynamic_cast<Object *>(t))
		obj->reference();
	return Glib::RefPtr<T>(t);
}

class UniChar {

public:
	UniChar() { unichar= gunichar(); }
	UniChar(gunichar ch) { unichar= ch; }
	operator gunichar () const { return unichar; }

private:
	gunichar unichar;

};

template <> Glib::ustring ToString<UniChar>(const UniChar & value);
template <> UniChar FromString<UniChar>(const Glib::ustring & str);

template <> UniChar ExtractCppValue<UniChar>(const Glib::ValueBase & val);
template <> Glib::ValueBase MakeGValue<UniChar>(const UniChar & val);

class Translatable {

public:
	Translatable(const Glib::ustring & meta=Glib::ustring());
	Translatable(bool trans, const Glib::ustring & prefix, const Glib::ustring & comments);

	bool getTranslate() const { return trans; }
	const Glib::ustring * getPrefix() const { return &prefix; }
	const Glib::ustring * getComments() const { return &comments; }
	bool isValid() const { return valid; }
	
	Glib::ustring getMeta() const;

protected:
	void testValid();

private:
	bool trans;
	Glib::ustring prefix;
	Glib::ustring comments;
	bool valid;

};

template <class Key> class Pool {

public:
	Pool(int cap) { capacity= cap; }

	PObject get(const Key & key) {
		typename Entries::iterator it= find(key);
		return it!=entries.end() ? it->second : PObject();
	}

	template <class T> Glib::RefPtr<T> get(const Key & key) {
		typename Entries::iterator it= find(key);
		return it!=entries.end() ? Glib::RefPtr<T>::cast_dynamic(it->second) : Glib::RefPtr<T>();
	}

	void set(const Key & key, PObject object) {
		typename Entries::iterator it= find(key);
		entries.push_back(std::make_pair(key, object));
		if(it==entries.end()) {
			if((int)entries.size() > capacity)
				entries.pop_front();
		} else
			entries.erase(it);
	}

protected:
	typedef std::list<std::pair<Key, PObject> > Entries;

	typename Entries::iterator find(const Key & key) {
		for(typename Entries::iterator it=entries.begin(); it!=entries.end(); ++it)
			if(it->first==key)
				return it;
		return entries.end();
	}

private:
	int capacity;
	Entries entries;

};

struct Point {
	int x, y;
	Point();
	Point(int x, int y);
	bool inside(const Rectangle & p) const;
	Point operator + (const Point & p) const;
	Point operator - (const Point & p) const;
	bool operator == (const Point & p) const;
	Point operator - () const;
};

template <> Glib::ustring ToString<Point>(const Point & value);
template <> Point FromString<Point>(const Glib::ustring & str);

struct Rectangle {
	int x, y, w, h;
	Rectangle();
	Rectangle(int x, int y, int w, int h);
	Rectangle(const Point & point, const Point & size);
	void grow(int dx, int dy, int dw, int dh);
	void grow(int dx, int dy, int dw, int dh, int minw, int minh);
	Point point() const { return Point(x, y); }
	Point size() const { return Point(w, h); }
	bool empty() const;
	Rectangle(const Gdk::Rectangle & rect);
	operator Gdk::Rectangle () const;
	bool operator == (const Rectangle & r) const;
	void growSegment(int * a, int * b, int da, int db, int min);
};

struct Border {
	int left, right, top, bottom;
	Border();
	bool operator == (const Border & b) const;
	operator Gtk::Border () const;
};

template <> Glib::ustring ToString<Border>(const Border & value);
template <> Border FromString<Border>(const Glib::ustring & str);

struct Emitter {
	std::string signame;
	std::string mapname;
	bool after;
	Emitter() { after= true; }
	Emitter(const std::string & sname, const std::string & mname, bool after);
	bool operator == (const Emitter & e) const;
};

template <> Glib::ustring ToString<Emitter>(const Emitter & value);
template <> Emitter FromString<Emitter>(const Glib::ustring & str);

class Id {

public:
	Id() { number= -1; }
	Id(const std::string & str) { SplitNumber(str, &string, &number); }	
	Id(int num) { number= num; }

	std::string toString() const { return string + (number==-1 ? Glib::ustring() : ToString(number)); }

	bool operator == (const Id & id) const { return string==id.string && number==id.number; }
	bool operator < (const Id & id) const { return string==id.string ? number < id.number : string < id.string; }

private:
	std::string string;
	int number;

};

class Parser {

public:
	Parser();
	virtual ~Parser() { }

	static void parse(Parser * parser, const Glib::ustring & data);

protected:
	typedef Glib::Markup::Parser::AttributeMap AttributeMap;

	virtual void onStartElement(const Glib::ustring & name, const AttributeMap & properties) { }
	virtual void onEndElement(const Glib::ustring & name) { }
	virtual void onText(const Glib::ustring & text) { }
	virtual void onPassthrough(const Glib::ustring & text) { }

private:

	class GlibParser : public Glib::Markup::Parser {

	public:
		GlibParser(::Crow::Parser * parser) { this->parser= parser; }

	protected:		
		void raiseError(const Glib::Exception & x);
		
		void on_start_element(Glib::Markup::ParseContext & context, const Glib::ustring & name, const AttributeMap & properties);
		void on_end_element(Glib::Markup::ParseContext & context, const Glib::ustring & name);
		void on_text(Glib::Markup::ParseContext & context, const Glib::ustring & text);
		void on_passthrough(Glib::Markup::ParseContext & context, const Glib::ustring & text);

	private:
		::Crow::Parser * parser;

	};

	GlibParser glibParser;

};

class UiElementExtract : public Parser {

public:
	typedef std::map<std::string, std::string> emap;
	
	emap extract(const Glib::ustring & str);

protected:
	void onStartElement(const Glib::ustring & name, const AttributeMap & properties);
	void onEndElement(const Glib::ustring & name) { --depth; }

private:
	int depth;
	emap result;

};

}

#endif
